/**
 * Copyright 2016 - present, Matthieu Jabbour <matthieu.jabbour@gmail.com>.
 * All rights reserved.
 */


/* eslint-disable import/no-unresolved */


const path = require('path');
const webpack = require('webpack');
const userConfig = require('../../../config');
const packageJson = require('../../../package.json');
const ClearTerminalPlugin = require('./ClearTerminalPlugin');


// webpack's development configuration.
const developmentConfig = {
  target: 'web',
  bail: false,
  cache: true,
  context: userConfig.srcPath,
  devtool: 'cheap-module-source-map',
  entry: {
    [packageJson.name]: [
      './main.js',
      // This entry is required to correctly enable HMR.
      'webpack-hot-middleware/client?dynamicPublicPath=true&timeout=4000&reload=true&path=_hmr',
    ],
  },
  output: {
    filename: '[name].js',
    publicPath: '/assets/',
    path: `${userConfig.publicPath}/assets`,
    pathinfo: true,
  },
  resolve: {
    extensions: ['.json', '.js', '.jsx', '.vue', '*'],
    // We include the Vue compiler along with the builder to allow for `template` syntax.
    alias: { vue: 'vue/dist/vue.runtime.js' },
  },
  module: {
    rules: [
      // `babel-loader` is used to provide polyfills for latest ECMAScript syntax.
      {
        test: /\.jsx?$/,
        include: [userConfig.srcPath],
        use: [
          {
            loader: 'babel-loader',
            options: {
              cacheDirectory: true,
              // We don't use the `.babelrc` since it is already used to compile
              // development environment's scripts, with another configuration than this one.
              babelrc: false,
              presets: [['env', { es2015: { modules: false } }]],
            },
          },
        ],
      },
      // `vue-loader` is used to compile *.vue components files.
      {
        test: /\.vue$/,
        include: [userConfig.srcPath],
        loader: 'vue-loader',
        options: {
          // We disable CSS extraction in dev mode to enable CSS HMR.
          extractCSS: false,
          loaders: {
            js: [
              // `babel-loader` is used to provide polyfills for latest ECMAScript syntax.
              {
                loader: 'babel-loader',
                options: {
                  cacheDirectory: false,
                  // We don't use the `.babelrc` since it is already used to compile
                  // development environment's scripts, with another configuration than this one.
                  babelrc: false,
                  presets: [['env', { es2015: { modules: false } }]],
                },
              },
            ],
            css: [
              // `vue-style-loader` turns CSS into JS modules.
              'vue-style-loader',
              // `css-loader` resolves paths in CSS and adds assets as dependencies.
              { loader: 'css-loader', options: { sourceMap: true, minimize: true } },
              // `postcss-loader` is used to autoprefix CSS to ensure compatibility with browsers.
              {
                loader: 'postcss-loader',
                options: {
                  config: { path: path.resolve(__dirname, 'postcss.config.js') },
                  sourceMap: true,
                },
              },
            ],
          },
        },
      },
      {
        // `eslint-loader` is used first, to lint the code.
        test: /\.(vue|jsx?)$/,
        include: [userConfig.srcPath],
        use: [
          {
            loader: 'eslint-loader',
            options: { cache: true },
          },
        ],
      },
      {
        test: /\.json$/,
        include: [userConfig.srcPath],
        use: [{ loader: 'json-loader' }],
      },
      {
        test: /\.css$/,
        use: [
          // `vue-style-loader` turns CSS into JS modules.
          {
            loader: 'vue-style-loader',
            options: {},
          },
          // `css-loader` resolves paths in CSS and adds assets as dependencies.
          {
            loader: 'css-loader',
            options: { sourceMap: true, import: false },
          },
          // `postcss-loader` is used to autoprefix CSS to ensure compatibility with all browsers.
          {
            loader: 'postcss-loader',
            options: {
              config: { path: path.resolve(__dirname, 'postcss.config.js') },
              sourceMap: true,
            },
          },
        ],
      },
      {
        exclude: [/\.css$/, /\.jsx?$/, /\.json$/, /\.vue$/],
        use: [
          // To process any other file than Javascript / JSX and CSS files, we use `url-loader`.
          {
            loader: 'url-loader',
            options: {
              limit: 10 * 1024,
              name: '[name].[ext]',
            },
          },
        ],
      },
    ],
  },
  plugins: [
    // Enables HMR with webpack-dev-middleware.
    new webpack.HotModuleReplacementPlugin(),
    // Assigns the module and chunk ids by occurrence count. Ids that are used often get lower
    // (shorter) ids. This make ids predictable, reduces total file size and is recommended.
    new webpack.optimize.OccurrenceOrderPlugin(true),
    // Handles errors more cleanly and prevents Webpack from outputting anything into a bundle.
    new webpack.NoEmitOnErrorsPlugin(),
    // Makes some environment variables available to the JS code.
    new webpack.DefinePlugin({
      'process.env.NODE_ENV': '"development"',
    }),
    // The following plugins split code in several chunks to leverage on long-term vendor-caching.
    new webpack.optimize.CommonsChunkPlugin({
      name: 'vendor',
      minChunks: (module, count) => (/node_modules/.test(module.context) || count === 2),
    }),
    new webpack.optimize.CommonsChunkPlugin({
      name: 'common',
      minChunks: module => (!/node_modules/.test(module.context)),
    }),
    new webpack.optimize.CommonsChunkPlugin({
      name: 'bootstrap',
      minChunks: Infinity,
    }),
    // Includes copyright comment on top of each generated file.
    new webpack.BannerPlugin({
      banner: userConfig.banner,
      raw: true,
    }),
    // Clears terminal between each compilation.
    new ClearTerminalPlugin(),
  ],
  node: {
    // Prevents webpack from injecting useless setImmediate polyfill because Vue
    // source contains it (although only uses it if it's native).
    setImmediate: false,
    // Prevents webpack from injecting mocks to Node native modules
    // that does not make sense for the client
    dgram: 'empty',
    fs: 'empty',
    net: 'empty',
    tls: 'empty',
    child_process: 'empty',
  },
};


module.exports = developmentConfig;
