/**
 * Copyright 2016 - present, Matthieu Jabbour <matthieu.jabbour@gmail.com>.
 * All rights reserved.
 */


/* eslint-disable import/no-unresolved */


const path = require('path');
const webpack = require('webpack');
const userConfig = require('../../../config');
const packageJson = require('../../../package.json');
const ClearTerminalPlugin = require('./ClearTerminalPlugin');
const ExtractTextPlugin = require('extract-text-webpack-plugin');


// webpack's production configuration.
const productionConfig = {
  target: 'web',
  bail: true,
  cache: false,
  context: userConfig.srcPath,
  devtool: 'source-map',
  performance: {
    hints: 'error',
  },
  entry: { [packageJson.name]: './main.js' },
  output: {
    filename: '[name].[chunkhash].js',
    path: `${userConfig.publicPath}/assets`,
    pathinfo: false,
  },
  resolve: {
    extensions: ['.json', '.js', '.jsx', '.vue', '*'],
    // We include the Vue compiler along with the builder to allow for `template` syntax.
    alias: { vue: 'vue/dist/vue.runtime.min.js' },
  },
  module: {
    rules: [
      // `babel-loader` is used to provide polyfills for latest ECMAScript syntax.
      {
        test: /\.jsx?$/,
        include: [userConfig.srcPath],
        use: [
          {
            loader: 'babel-loader',
            options: {
              cacheDirectory: false,
              // We don't use the `.babelrc` since it is already used to compile
              // development environment's scripts, with another configuration than this one.
              babelrc: false,
              presets: [['env', { es2015: { modules: false } }]],
            },
          },
        ],
      },
      // `vue-loader` is used to compile *.vue components files.
      {
        test: /\.vue$/,
        loader: 'vue-loader',
        options: {
          hotReload: false, // disables Hot Reload.
          extractCSS: true,
          loaders: {
            js: [
              // `babel-loader` is used to provide polyfills for latest ECMAScript syntax.
              {
                loader: 'babel-loader',
                options: {
                  cacheDirectory: false,
                  // We don't use the `.babelrc` since it is already used to compile
                  // development environment's scripts, with another configuration than this one.
                  babelrc: false,
                  presets: [['env', { es2015: { modules: false } }]],
                },
              },
            ],
            css: ExtractTextPlugin.extract({
              // `vue-style-loader` turns CSS into JS modules.
              fallback: 'vue-style-loader',
              use: [
                // `css-loader` resolves paths in CSS and adds assets as dependencies.
                { loader: 'css-loader', options: { sourceMap: true, minimize: true } },
                // `postcss-loader` is used to autoprefix CSS to ensure compatibility with browsers.
                {
                  loader: 'postcss-loader',
                  options: {
                    config: { path: path.resolve(__dirname, 'postcss.config.js') },
                    sourceMap: true,
                  },
                },
              ],
            }),
          },
        },
      },
      // `eslint-loader` is used first, to lint the code.
      {
        test: /\.(vue|jsx?)$/,
        include: [userConfig.srcPath],
        use: [
          {
            loader: 'eslint-loader',
            options: { cache: true },
          },
        ],
      },
      {
        test: /\.json$/,
        include: [userConfig.srcPath],
        use: [{ loader: 'json-loader' }],
      },
      {
        test: /\.css$/,
        use: ExtractTextPlugin.extract({
          // `vue-style-loader` turns CSS into JS modules.
          fallback: 'vue-style-loader',
          use: [
            // `css-loader` resolves paths in CSS and adds assets as dependencies.
            { loader: 'css-loader', options: { sourceMap: true, minimize: true } },
            // `postcss-loader` is used to autoprefix CSS to ensure compatibility with all browsers.
            {
              loader: 'postcss-loader',
              options: {
                config: { path: path.resolve(__dirname, 'postcss.config.js') },
                sourceMap: true,
              },
            },
          ],
        }),
      },
      {
        exclude: [/\.css$/, /\.jsx?$/, /\.json$/, /\.vue$/],
        use: [
          // To process any other file than Javascript / JSX and CSS files, we use `url-loader`.
          {
            loader: 'url-loader',
            options: {
              limit: 10 * 1024,
              name: '[name].[ext]',
            },
          },
        ],
      },
    ],
  },
  plugins: [
    // Extracts all the CSS into a specific file.
    new ExtractTextPlugin('[name].[chunkhash].css'),
    // Assigns the module and chunk ids by occurrence count. Ids that are used often get lower
    // (shorter) ids. This make ids predictable, reduces total file size and is recommended.
    new webpack.optimize.OccurrenceOrderPlugin(true),
    // Handles errors more cleanly and prevents Webpack from outputting anything into a bundle.
    new webpack.NoEmitOnErrorsPlugin(),
    // Makes some environment variables available to the JS code.
    new webpack.DefinePlugin({
      'process.env.NODE_ENV': '"production"',
    }),
    new webpack.optimize.UglifyJsPlugin({
      sourceMap: true,
      uglifyOptions: {
        ecma: 6,
      },
    }),
    // The following plugins split code in several chunks to leverage on long-term vendor-caching.
    new webpack.optimize.CommonsChunkPlugin({
      name: 'vendor',
      minChunks: (module, count) => (/node_modules/.test(module.context) || count === 2),
    }),
    new webpack.optimize.CommonsChunkPlugin({
      name: 'common',
      minChunks: module => (!/node_modules/.test(module.context)),
    }),
    new webpack.optimize.CommonsChunkPlugin({
      name: 'bootstrap',
      minChunks: Infinity,
    }),
    // Includes copyright comment on top of each generated file.
    new webpack.BannerPlugin({
      banner: userConfig.banner,
      raw: true,
    }),
    // Clears terminal between each compilation.
    new ClearTerminalPlugin(),
  ],
  node: {
    // prevent webpack from injecting useless setImmediate polyfill because Vue
    // source contains it (although only uses it if it's native).
    setImmediate: false,
    // prevent webpack from injecting mocks to Node native modules
    // that does not make sense for the client
    dgram: 'empty',
    fs: 'empty',
    net: 'empty',
    tls: 'empty',
    child_process: 'empty',
  },
};


module.exports = productionConfig;
